using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace kivie
{	

	public class MorphClone : MVRScript
	{
		#region PluginInfo    
        public string pluginAuthor = "kivie";
        public string pluginName = "MorphClone";
        public string pluginVersion = "1.0";
        public string pluginDate = "2022-01-19";
        public string pluginDescription = "Copy morphs from one person atom to another";
        #endregion

        protected Atom CurrentAtom;
        protected GenerateDAZMorphsControlUI CurrentMorphControl;

        protected Atom TargetAtom;
        protected GenerateDAZMorphsControlUI TargetMorphControl;

		public override void Init()
		{
			try
			{
                // Save our current atom; Redundant
                CurrentAtom = containingAtom;

                // Create a list of other Person targets for a drop down
                List<string> targetChoices = new List<string>();
                foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
                {
                    TargetAtom = SuperController.singleton.GetAtomByUid(atomUID);
                    if (TargetAtom != containingAtom && TargetAtom.type == "Person")
                    {
                        targetChoices.Add(atomUID);
                    }
                }
                UIDynamicPopup udp = CreatePopup(new JSONStorableStringChooser("Target", targetChoices, "None", "Target Person", this.TargetChange), false);

			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught: " + e);
			}
		}
		
        // Call back for target selection
        void TargetChange(string val)
        {
            SuperController.LogMessage("Source Change: " + val);
            if (!val.Equals("None")) {
                TargetAtom = SuperController.singleton.GetAtomByUid(val);
                TargetMorphControl = GetMorphControl(TargetAtom);
            } else {
                TargetAtom = null;
                TargetMorphControl = null;
            }
        }        

		void Start()
		{
			try
			{
                CurrentMorphControl = GetMorphControl(CurrentAtom);       
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught in Start(): " + e);
			}
		}

        private GenerateDAZMorphsControlUI GetMorphControl(Atom atom)
        {
            JSONStorable geometry = atom.GetStorableByID("geometry");
            DAZCharacterSelector character = geometry as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = character.morphsControlFemaleUI;
            // GenerateDAZMorphsControlUI morphControl = character.morphsControlUI;

            return morphControl;
        }

		void Update()
		{
			try
			{
                if (TargetMorphControl != null) {
                    IList<DAZMorph> morphList = CurrentMorphControl.GetMorphs();
                    morphList = morphList.Where(x => x.active && !x.isDriven).ToList();
                    
                    foreach (DAZMorph morph in morphList) {
                        if (morph.displayName.Contains("G2Female")) {
                            continue;
                        }

						DAZMorph targetMorph = TargetMorphControl.GetMorphByUid(morph.uid);
                        if (targetMorph != null) {
                            targetMorph.SetValue(morph.morphValue);
                        } else {
                            SuperController.LogMessage("Could not find: " + morph.uid);
                        }
                    }
                }
			}
			catch (Exception e)
			{
				SuperController.LogError("Exception caught in Update(): " + e);
			}
		}
	}
}